/***************************************************************************
                          knob.cpp  -  description
                             -------------------
    begin                : sob lip  6 23:48:47 CEST 2002
    copyright            : (C) 2002 by Krzysztof Lichota
    email                : lichota@mimuw.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapp.h>
#include <kmessagebox.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kaction.h>

#include "knob.h"
#include "knobdial.h"

extern "C"
{
  KPanelApplet* init( QWidget *parent, const QString configFile)
  {
    KGlobal::locale()->insertCatalogue("Knob");
    return new Knob(configFile, KPanelApplet::Normal,
                      KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
                      parent, "Knob");
  }
}

Knob::Knob(const QString& configFile, Type type, int actions, QWidget *parent, const char *name) 
       : KPanelApplet(configFile, type, actions, parent, name)
{
	// Get the current application configuration handle
	ksConfig = config();

	//show main view
	mainView = new KnobMainView(this);
	mainView->setGeometry(QRect(0, 0, width(), height()));

	this->volumeControl = new KnobVolumeCtrl_OSS;

	connect(mainView->knobDial, SIGNAL(valueChanged(int)),  SLOT(changeVolume(int)));

	connect(mainView->knobDial, SIGNAL(middleClick()),  SLOT(switchMute()));

	if (this->updateTimer.start(VOLUME_UPDATE_INTERVAL) == 0)
	{
		KMessageBox::error(this, i18n("Starting timer failed, volume state will not be updated."));
	}
	else
	{
		connect(&this->updateTimer, SIGNAL(timeout()), SLOT(updateVolume()));
	}
        
	mainView->show();
}

Knob::~Knob()
{
}

void Knob::changeVolume(int volume)
{
	if (this->volumeControl->isMuted())
	{
		return;
	}
		
	bool result = this->volumeControl->setVolume(volume);
	if (result == false)
	{
		//error
		this->updateTimer.stop();
		
		KMessageBox::error(this, i18n("Error when trying to set volume on mixer. Disabling volume updating.\n"
			"Please remove Knob and add it again to panel when mixer is set up correctly."));
		return;
	}
}

void Knob::updateVolume()
{
	int volume = 	this->volumeControl->getVolume();

	if (volume == -1)
	{
		//error
		this->updateTimer.stop();
		
		KMessageBox::error(this, i18n("Error when trying to get volume from mixer. Disabling volume updating.\n"
			"Please remove Knob and add it again to panel when mixer is set up correctly."));

		return;
	}

	if (this->volumeControl->isMuted() && volume != 0)
	{
		this->setMuted(false);
	}

			
	this->mainView->knobDial->setValue(volume);
}

static const char* description=I18N_NOOP("Simple volume control applet for KDE");
static const char* version="1.2";

void Knob::about()
{
//   KMessageBox::information(0, i18n("This is an about box"));

	KAboutData about(
		"Knob",
		I18N_NOOP("Knob"),
		version,
		description,
		KAboutData::License_GPL,
		I18N_NOOP("(c) 2002, Krzysztof Lichota")
		);

	about.addAuthor("Krzysztof Lichota", NULL, "lichota@mimuw.edu.pl");

	//credits
	about.addCredit("Albert \"Tigr\" Dorofeev and John \"wizgrav\" Gravezas", "Creators of asmix");

	KAboutApplication aboutDialog(&about, this, 0);
	aboutDialog.exec();
}

void Knob::help()
{
   KMessageBox::information(0, i18n("This is a help box"));
}

void Knob::preferences()
{
	KMessageBox::information(0, i18n("No preferences yet."));
	return;
	
	KnobDial *knob;

	knob = new KnobDial();
	knob->setMinimumSize(QSize(1,1));
	knob->show();
}

int Knob::widthForHeight( int height ) const
{
//   return width();
//	return this->height();
	return height;
}

int Knob::heightForWidth( int width ) const
{
//   return height();
//	return this->width();
	return width;
}

void Knob::resizeEvent( QResizeEvent *_Event)
{
	mainView->setGeometry( QRect( 0, 0, _Event->size().width(), _Event->size().height()) );
}

void KnobMainView::setGeometry(const QRect& rect)
{
//	myview::setGeometry(rect);

	QRect knobRect;
	int size = QMIN(rect.width(), rect.height());

	knobRect.setRect(rect.left(), rect.top(), size, size);
	
	this->knobDial->setGeometry(knobRect);
}

void Knob::switchMute()
{
	if (this->volumeControl->isMuted())
	{
		this->setMuted(false);
	}
	else
	{
		this->setMuted(true);
	}
}

void Knob::setMuted(bool mute)
{
	if (mute)
	{
		//mute
		this->volumeControl->mute();
		
		KnobDialProperties props = this->mainView->knobDial->getDialProperties();
		props.drawCross = true;
		props.crossColor = green;
		this->mainView->knobDial->setDialProperties(props);
		this->mainView->knobDial->repaint();
	}
	else
	{
		//unmute
		this->volumeControl->unMute();
		
		KnobDialProperties props = this->mainView->knobDial->getDialProperties();
		props.drawCross = false;
		this->mainView->knobDial->setDialProperties(props);
		this->mainView->knobDial->repaint();
	}
}
