/***************************************************************************
                          knobdial.cpp  -  description
                             -------------------
    begin                : Sun Jul 7 2002
    copyright            : (C) 2002 by Krzysztof Lichota
    email                : lichota@mimuw.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "knobdial.h"

#include <kimageeffect.h>
#include <qimage.h>
#include <kmessagebox.h>
#include <qpainter.h>

KnobDial::KnobDial(QWidget *parent, const char *name):QDial(parent, name)
{
	this->properties.drawCross = false;
}

void KnobDial::mouseReleaseEvent(QMouseEvent *e)
{
	if (e->button() == QMouseEvent::MidButton)
	{
//		KMessageBox::information(NULL, "Click");
		e->accept();
		emit this->middleClick();
	}
	else
	{
		QDial::mouseReleaseEvent(e);
	}
}

void KnobDial::mousePressEvent(QMouseEvent *e)
{
	if (e->button() == QMouseEvent::MidButton)
	{
		e->accept();
	}
	else
	{
		QDial::mousePressEvent(e);
	}
}

void KnobDial::repaintScreen( const QRect *cr )
{
    QPainter p;
    QImage outerEllipseImage;
    QImage innerEllipseImage;
    int outerOffset = 2;
    int innerOffset;
    int innerOffsetPercentage;
    QRect outerEllipseRect(outerOffset, outerOffset, width()-2*outerOffset, height()-2*outerOffset);

    QPixmap pixmap(width(), height());

    p.begin(&pixmap);    
//    p.begin( this );

    innerOffset = 3;
    innerOffsetPercentage = 15;
    innerOffset =(QMIN(outerEllipseRect.width(), outerEllipseRect.height()) * innerOffsetPercentage)/(100 * 2/*the same as width()/2*/);
    innerOffset = QMAX(innerOffset, 1);

    QRect innerEllipseRect = QRect(outerEllipseRect.x() + innerOffset, outerEllipseRect.y() + innerOffset, outerEllipseRect.width()-2*innerOffset, outerEllipseRect.height()-2*innerOffset);

    p.setPen( NoPen );
    p.setBrush(lightGray);
    outerEllipseImage = KImageEffect::gradient(outerEllipseRect.size(), white, black,KImageEffect::DiagonalGradient, 0);
    innerEllipseImage = KImageEffect::gradient(innerEllipseRect.size(), lightGray, darkGray,KImageEffect::DiagonalGradient, 0);

//    te.setWidth(te.width());
//    te.setHeight(te.height());
    // erase background of dial
//	p.drawEllipse( te );

    // erase remaining space around the dial
    p.save();
    QRegion remaining( 0, 0, width(), height() );
    remaining = remaining.subtract( QRegion( outerEllipseRect, QRegion::Ellipse ) );

    if ( p.hasClipping() )
		remaining = remaining.intersect( p.clipRegion() );

    p.setClipRegion( remaining );
    p.fillRect( remaining.boundingRect(), colorGroup().brush( QColorGroup::Background ) );
    p.restore();

   // draw bigger ellipse
    p.save();

    remaining = QRegion( outerEllipseRect, QRegion::Ellipse );

    if ( p.hasClipping() )
		remaining = remaining.intersect( p.clipRegion() );

    p.setClipRegion( remaining );
	p.drawImage(outerEllipseRect.x(), outerEllipseRect.y(), outerEllipseImage);

	p.restore();

	// draw smaller ellipse
    p.save();

    remaining = QRegion( innerEllipseRect, QRegion::Ellipse );

    if ( p.hasClipping() )
		remaining = remaining.intersect( p.clipRegion() );

    p.setClipRegion( remaining );
	p.drawImage(innerEllipseRect.x(), innerEllipseRect.y(), innerEllipseImage);
	p.restore();

	//draw the dot
	p.save();
	double degrees;
	double a;

//    if ( p.hasClipping() )
//    		p.setClipRegion( p.clipRegion() );
	p.setWindow( -1000,-1000, 2000, 2000 );
	p.setViewport(innerEllipseRect);

       QRect v = innerEllipseRect;
        int d = QMIN( v.width(), v.height() );

//        p.setViewport( v.left() + (v.width()-d)/2,
//                           v.top() + (v.height()-d)/2, d, d );

	int offset = 30;

	a = ((double)value() - (double)minValue()) / ((double)maxValue() - (double)minValue());
	degrees = (a * (360 - 2*offset)) - 270+offset;
	p.rotate(degrees);

//	p.setPen(QPen(black, 1));
//	p.drawLine(0,0, 100, 0);
//	p.drawLine(0, 0, 0, 100);

	p.setBrush(darkRed);
//	p.setPen(QPen(black,1));
	p.setPen(NoPen);
//	p.drawEllipse(400, -75, 150, 150);
	int pointRadius;

	pointRadius = 75;
	p.drawEllipse(600-pointRadius, -pointRadius, 2*pointRadius, 2*pointRadius);

//	p.drawEllipse(-1000, -200, -500, 200);

	p.restore();
	
	if (this->properties.drawCross)
	{
		//draw cross
		p.save();
		
		p.setWindow( -1000,-1000, 2000, 2000 );
		p.setViewport(outerEllipseRect);

		QPen pen;
		pen.setColor(green);
		pen.setWidth(4);

		p.setPen(pen);

		int span = 400;

		p.drawLine(-span,-span, span, span);
		p.drawLine(-span, span, span, -span);
		p.restore();
	}
	
    // calculate and paint arrow

//    p.setPen( QPen( colorGroup().light(), 2 ) );
//    p.drawArc( te, 60 * 16, 180 * 16 );

//    p.setPen( QPen( colorGroup().dark(), 2 ) );
//    p.drawArc( te, 240 * 16, 180 * 16 );


	// strange, but else we get redraw errors on windows!!
	p.end();

	p.begin(&pixmap);
	p.save();
	p.setPen( QPen( colorGroup().background() ) );
	p.setBrush( NoBrush );
//	p.drawRect( br );
	p.restore();
//	style().drawFocusRect( &p, br, colorGroup() , &colorGroup().background() );
	p.end();

	QPainter widgetPainter;

	widgetPainter.begin(this);

	// calculate clip-region for erasing background
	if ( cr ) {
		p.setClipRect( *cr );
	}
	
	widgetPainter.drawPixmap(0, 0, pixmap);

	widgetPainter.end();
}
