/***************************************************************************
                          volume.cpp  -  description
                             -------------------
    begin                : Thu Jul 18 2002
    copyright            : (C) 2002 by Krzysztof Lichota
    email                : lichota@mimuw.edu.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "volume.h"

#include <linux/soundcard.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>

#include <qstring.h>
#include <kmessagebox.h>
#include <klocale.h>

KnobVolumeCtrl::KnobVolumeCtrl()
{
	this->muted = false;
}

bool KnobVolumeCtrl::isMuted()
{
	return this->muted;
}

void KnobVolumeCtrl::mute()
{
	if (isMuted())
	{
		return;
	}
	this->unmutedVolume = this->getVolume();
	this->muted = true;
	this->setVolume(0);
}

void KnobVolumeCtrl::unMute()
{
	if (!isMuted())
	{
		return;
	}
	this->muted = false;
	this->setVolume(this->unmutedVolume);
}

//OSS control code taken from asmixer

bool KnobVolumeCtrl_OSS::setVolume(int volume)
{
	if (this->openMixer() == false)
	{
		return false;
	}

	this->volControl.volume.left=this->volControl.volume.right=(unsigned char)volume;

	if (ioctl(this->mixerFd, MIXER_WRITE(this->volControl.mixer_id), &this->volControl.volume) == -1)
	{
		this->error = true;
		KMessageBox::error(0, i18n("Cannot perform ioctl() on mixer device"));
		return false;
	}

	this->closeMixer();

	return true;
}

int KnobVolumeCtrl_OSS::getVolume()
{
	if (this->openMixer() == false)
	{
		return -1;
	}

	if (ioctl(this->mixerFd, MIXER_READ(this->volControl.mixer_id), &this->volControl.volume) == -1)
	{
		this->error = true;
		KMessageBox::error(0, i18n("Cannot perform ioctl() on mixer device"));
		return -1;
	}

	this->closeMixer();
	
	int volume = ((int)this->volControl.volume.left + (int)this->volControl.volume.right)/2;

	return volume;
}

bool KnobVolumeCtrl_OSS::openMixer()
{
	if (this->error)
	{
		return false;
	}

	//FIXME: get device name from configuration
	QString mixerDeviceName = "/dev/mixer";
	
	this->mixerFd = open(mixerDeviceName, O_RDWR, 0);

	if (this->mixerFd == -1)
	{
		this->error = true;
		KMessageBox::error(0, i18n("Cannot open mixer device\"%1\"").arg(mixerDeviceName));
		return false;
	}

	if (ioctl(this->mixerFd, SOUND_MIXER_READ_DEVMASK, &this->volControl.supported) == -1)
	{
		this->volControl.supported = 0xffff; // Assume all are supported
	}

	this->volControl.mixer_id = SOUND_MIXER_VOLUME;

	return true;
}

void KnobVolumeCtrl_OSS::closeMixer()
{
	::close(this->mixerFd);
}
